### structured example for Analogs.R

library(CSTools)
library(startR)
library(s2dv)
library(lubridate)
library(multiApply)
library(FNN)
source('R/Interpolation.R')
source('R/Analogs.R')
source('R/Utils.R')

# prescribe extend 
lonmin <- -11.5
lonmax <- 5.35
latmin <- 35.1
latmax <- 44.1
extra <- 1

# define sdates
sdates_exp <- format(ymd("20000501") + rep(years(0:8), each=1),"%Y%m%d")
sdates_obs  <- format(ymd("20000501") + rep(years(0:8), each=1),"%Y%m")
sdates_fcst <- format(seq(ymd("20210201"), ymd("20210601"), "1 month"), "%Y%m%d")

#---------------------------
# Observations
#---------------------------
# ERA5-Land observations: high-resolution observations
# daily observations
obs_hres <- startR::Start(dat = '/esarchive/recon/ecmwf/era5land/daily_mean/$var$_f1h/$var$_$sdate$.nc',
                          var = 'tas', time = indices(1:28), latitude = values(list(latmin - extra, latmax + extra)),
                          sdate = sdates_obs, latitude_reorder = Sort(decreasing = FALSE),
                          longitude = values(list(lonmin - extra, lonmax + extra)),
                          longitude_reorder = CircularSort(-180, 180),
                          synonims = list(var = c('var','variable'), longitude = c('lon', 'longitude'),
                                          latitude = c('lat', 'latitude')), return_vars = list(latitude = 'dat', longitude = 'dat'),
                          num_procs = 1, retrieve = TRUE)

# daily obs from era5 
obs_era5 <- startR::Start(dat = '/esarchive/recon/ecmwf/era5/daily_mean/$var$_f1h/$var$_$sdate$.nc',
                          var = 'tas', time = indices(1:28), latitude = values(list(latmin - extra, latmax + extra)), 
                          sdate = sdates_obs, latitude_reorder = Sort(decreasing = FALSE),
                          longitude = values(list(lonmin - extra, lonmax + extra)), 
                          longitude_reorder = CircularSort(-180, 180),
                          synonims = list(var = c('var','variable'), longitude = c('lon', 'longitude'), 
                                          latitude = c('lat', 'latitude')), return_vars = list(latitude = 'dat', longitude = 'dat'),
                          num_procs = 1, retrieve = TRUE)

# Create 's2dv_cube' object
obs_s2dv <- as.s2dv_cube(obs_era5) 

#---------------------------
# Model
#---------------------------
# ecmwf- system5c3s daily coarse resolution
exp <- startR::Start(dat = '/esarchive/exp/ecmwf/system5c3s/daily_mean/$var$_f6h/$var$_$sdate$.nc',
                     var = 'tas', time = indices(1:28), member = indices(1:3), sdate = sdates_exp,
                     latitude = values(list(latmin, latmax)), latitude_reorder = Sort(decreasing = FALSE), 
                     longitude = values(list(lonmin, lonmax)), longitude_reorder = CircularSort(-180, 180),
                     synonims = list(var = c('var','variable'), longitude = c('lon', 'longitude'), 
                                     latitude = c('lat', 'latitude'), member = c('member','ensemble')),
                     return_vars = list(latitude = 'dat', longitude = 'dat'),
                     num_procs = 1, retrieve = TRUE)

# Create 's2dv_cube' object
exp_s2dv <- as.s2dv_cube(exp)

#---------------------------
# Forecast
#---------------------------
fcst <- startR::Start(dat = '/esarchive/exp/ecmwf/system5c3s/monthly_mean/$var$_f6h/$var$_$sdate$.nc',
                      var = 'tas', time = indices(1), member = indices(1:3), sdate = sdates_fcst,
                      latitude = values(list(latmin, latmax)), latitude_reorder = Sort(decreasing = FALSE), 
                      longitude = values(list(lonmin, lonmax)), longitude_reorder = CircularSort(-180, 180),
                      synonims = list(var = c('var','variable'), longitude = c('lon', 'longitude'), 
                                      latitude = c('lat', 'latitude'), member = c('member','ensemble')),
                      return_vars = list(latitude = 'dat', longitude = 'dat'),
                      num_procs = 1, retrieve = TRUE)


# Simple parameter usage 
#---------------------------
# Analogs with no 'window' dimension in the initial object and with cv
#---------------------------

ana_1 <- Analogs(exp = exp, obs = obs_hres, exp_lats = attr(exp, "Variables")$dat1$lat,
                 exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs_hres, "Variables")$dat1$lat,
                 obs_lons = attr(obs_hres, "Variables")$dat1$lon, grid_exp = attr(exp, "Files")[1,1,1],
                 fun_analog = NULL, lat_dim = 'latitude', lon_dim = 'longitude', loocv_window = TRUE, ncores = 4)


#---------------------------
# Analogs with no 'window' dimension in the initial object and without cv
#---------------------------

ana_2 <- Analogs(exp = exp, obs = obs_era5, exp_lats = attr(exp, "Variables")$dat1$lat,
                 exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs_era5, "Variables")$dat1$lat,
                 obs_lons = attr(obs_era5, "Variables")$dat1$lon, grid_exp = attr(exp, "Files")[1,1,1],
                 fun_analog = NULL, lat_dim = 'latitude', lon_dim = 'longitude', loocv_window = FALSE, ncores = 4)


#---------------------------
# Analogs with 'window' dimension in the initial object and with cv
#---------------------------
# generate 'window' dimension

obs_window <- .generate_window(obj = obs_era5, sdate_dim = 'sdate', time_dim = 'time', loocv = TRUE) 
dim(obs_window) <- c(dim(obs_window), time =1) # bcs time dim is compromised when making window

ana_3 <- Analogs(exp = exp, obs = obs_window, exp_lats = attr(exp, "Variables")$dat1$lat,
                 exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs_era5, "Variables")$dat1$lat,
                 obs_lons = attr(obs_era5, "Variables")$dat1$lon, grid_exp = attr(exp, "Files")[1,1,1],
                 fun_analog = NULL, lat_dim = 'latitude', lon_dim = 'longitude',  loocv_window = TRUE, ncores = 4)

#---------------------------
# Analogs with cv and returning the indices
#---------------------------

ana_4 <- Analogs(exp = exp, obs = obs_era5, exp_lats = attr(exp, "Variables")$dat1$lat,
                 exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs_era5, "Variables")$dat1$lat,
                 obs_lons = attr(obs_era5, "Variables")$dat1$lon, grid_exp = attr(exp, "Files")[1,1,1],
                 fun_analog = NULL, lat_dim = 'latitude', lon_dim = 'longitude', loocv_window = TRUE, 
                 return_indices = TRUE, ncores = 4)


#---------------------------
# Analogs with 'window' dimension and cv and returning the indices
#---------------------------
ana_5 <- Analogs(exp = exp, obs = obs_window, exp_lats = attr(exp, "Variables")$dat1$lat,
                 exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs_era5, "Variables")$dat1$lat,
                 obs_lons = attr(obs_era5, "Variables")$dat1$lon, grid_exp = attr(exp, "Files")[1,1,1],
                 fun_analog = NULL, lat_dim = 'latitude', lon_dim = 'longitude', loocv_window = TRUE,
                 return_indices = TRUE, ncores = 4)

#---------------------------
# Analogs with NA's in obs
#---------------------------

obs_era5[,,,10:20,,] <- NA
ana_obsna_mean <- Analogs(exp = exp, obs = obs_era5, exp_lats = attr(exp, "Variables")$dat1$lat,
                          exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs_era5, "Variables")$dat1$lat,
                          obs_lons = attr(obs_era5, "Variables")$dat1$lon, grid_exp = attr(exp, "Files")[1,1,1],
                          fun_analog = 'mean', ncores = 4,lat_dim='latitude',lon_dim='longitude')

#---------------------------
# Analogs with NA's in exp
#---------------------------
exp[,,,,,3:6,] <- NA
ana_expna_mean <- Analogs(exp = exp, obs = obs_era5, exp_lats = attr(exp, "Variables")$dat1$lat,
                          exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs_era5, "Variables")$dat1$lat,
                          obs_lons = attr(obs_era5, "Variables")$dat1$lon, grid_exp = attr(exp, "Files")[1,1,1],
                          fun_analog = 'mean', ncores = 4,lat_dim='latitude',lon_dim='longitude')

#---------------------------
# Analogs with specified region 
#---------------------------
crplonmin <- -1
crplonmax <- 4 
crplatmin <- 39
crplatmax <- 44

region_ext = c(crplonmin, crplonmax, crplatmin, crplatmax)

ana_region <- Analogs(exp = exp, obs = obs_hres, exp_lats = attr(exp, "Variables")$dat1$lat,
                      exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs_hres, "Variables")$dat1$lat,
                      obs_lons = attr(obs_hres, "Variables")$dat1$lon, grid_exp = attr(exp, "Files")[1,1,1],
                      fun_analog = NULL, lat_dim = 'latitude', lon_dim = 'longitude', 
                      loocv_window = TRUE, region = region_ext,
                      return_indices = TRUE, ncores = 4)

#------------------------------------------------------
# Analogs with large-scale observation
#------------------------------------------------------
ana_obsL <- Analogs(exp = exp, obs = obs_hres, obsL = obs_era5, exp_lats = attr(exp, "Variables")$dat1$lat,
                    exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs_hres, "Variables")$dat1$lat,
                    obs_lons = attr(obs_hres, "Variables")$dat1$lon, obs2_lats = attr(obs_era5, "Variables")$dat1$lat, 
                    obs2_lons = attr(obs_era5, "Variables")$dat1$lon, grid_exp = attr(exp, "Files")[1,1,1],
                    fun_analog = NULL, lat_dim = 'latitude', lon_dim = 'longitude', loocv_window = TRUE, 
                    return_indices = TRUE, ncores = 4)

#---------------------------
# Analogs with forecast
#---------------------------

ana_fcst1 <- Analogs(exp = fcst, obs = obs_hres, exp_lats = attr(exp, "Variables")$dat1$lat,
                 exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs_hres, "Variables")$dat1$lat, 
                 obs_lons = attr(obs_hres, "Variables")$dat1$lon,  grid_exp = attr(exp, "Files")[1,1,1], 
                 lat_dim = 'latitude', lon_dim = 'longitude', sdate_dim = 'sdate', 
                 time_dim = 'time', member_dim = 'member', return_indices = TRUE, 
                 loocv_window = TRUE, ncores = 4)





#-----------------------------------
# Analogs with monthly resolution and large-scale observation
#-----------------------------------

# prescribe extend 
lonmin <- -30
lonmax <- 25
latmin <- 30
latmax <- 65
extra <- 1

# monthly sdates
sdates_exp <- format(ymd("20000501") + rep(years(0:8), each=1),"%Y%m%d")
sdates_obs  <- format(ymd("20000501") + rep(years(0:8), each=1),"%Y%m")

obs_era5 <- startR::Start(dat = '/esarchive/recon/ecmwf/era5/monthly_mean/$var$_f1h/$var$_$sdate$.nc',
                          var = 'tas', time = indices(1), latitude = values(list(latmin - extra, latmax + extra)), 
                          sdate = sdates_obs, latitude_reorder = Sort(decreasing = FALSE),
                          longitude = values(list(lonmin - extra, lonmax + extra)), 
                          longitude_reorder = CircularSort(-180, 180),
                          synonims = list(var = c('var','variable'), longitude = c('lon', 'longitude'), 
                                          latitude = c('lat', 'latitude')), return_vars = list(latitude = 'dat', longitude = 'dat'),
                          num_procs = 1, retrieve = TRUE)

obs_s2dv <- as.s2dv_cube(obs_era5) 


obs_hres <- startR::Start(dat = '/esarchive/recon/ecmwf/era5land/monthly_mean/$var$_f1h/$var$_$sdate$.nc',
                          var = 'tas', time = indices(1), latitude = values(list(latmin - extra, latmax + extra)),
                          sdate = sdates_obs, latitude_reorder = Sort(decreasing = FALSE),
                          longitude = values(list(lonmin - extra, lonmax + extra)),
                          longitude_reorder = CircularSort(-180, 180),
                          synonims = list(var = c('var','variable'), longitude = c('lon', 'longitude'),
                                          latitude = c('lat', 'latitude')), return_vars = list(latitude = 'dat', longitude = 'dat'),
                          num_procs = 1, retrieve = TRUE)

exp <- startR::Start(dat = '/esarchive/exp/ecmwf/system5c3s/monthly_mean/$var$_f6h/$var$_$sdate$.nc',
                     var = 'tas', time = indices(1), member = indices(1:3), sdate = sdates_exp,
                     latitude = values(list(latmin, latmax)), latitude_reorder = Sort(decreasing = FALSE), 
                     longitude = values(list(lonmin, lonmax)), longitude_reorder = CircularSort(-180, 180),
                     synonims = list(var = c('var','variable'), longitude = c('lon', 'longitude'), 
                                     latitude = c('lat', 'latitude'), member = c('member','ensemble')),
                     return_vars = list(latitude = 'dat', longitude = 'dat'),
                     num_procs = 1, retrieve = TRUE)



ana_obsL <- Analogs(exp = exp, obs = obs_hres, obsL = obs_era5, exp_lats = attr(exp, "Variables")$dat1$lat,
                    exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs_hres, "Variables")$dat1$lat,
                    obs_lons = attr(obs_hres, "Variables")$dat1$lon, obsL_lats = attr(obs_era5, "Variables")$dat1$lat, 
                    obsL_lons = attr(obs_era5, "Variables")$dat1$lon, grid_exp = attr(exp, "Files")[1,1,1],
                    fun_analog = NULL, lat_dim = 'latitude', lon_dim = 'longitude', loocv_window = TRUE, 
                    return_indices = TRUE, ncores = 4)



#-----------------------------------
# Plots
#-----------------------------------
Cairo::CairoPNG("ana_null.png")
s2dv::PlotEquiMap(ana_1$data[,,1,1,1,1,1,1], lat = ana_1$lat, 
                  lon = ana_1$lon, filled.continents = F,
                  toptitle = 'Analogs NULL', brks = seq(275,288,1), height = 8, width = 10)
dev.off()

Cairo::CairoPNG("ana_obsna_mean.png")
s2dv::PlotEquiMap(ana_obsna_mean$data[,,1,1,1,1,1], lat = ana_obsna_mean$lat, 
                  lon = ana_obsna_mean$lon, filled.continents = F,
                  toptitle = 'Analogs mean', brks = seq(275,288,1), height = 8, width = 10)
dev.off()

Cairo::CairoPNG("ana_region.png")
s2dv::PlotEquiMap(ana_region$data[,,1,1,1,1,1,1], lat = ana_region$lat[ana_region$lat >= crplatmin &  ana_region$lat <= crplatmax], 
                  lon = ana_region$lon[ana_region$lon >= crplonmin &  ana_region$lon <= crplonmax], 
                  filled.continents = F,
                  toptitle = 'Analogs region', brks = seq(275,288,1), height = 8, width = 10)
dev.off()


#-----------------------------------
# Analogs with CST_Analogs()
#-----------------------------------
cstana_max <- CST_Analogs(exp = exp_s2dv, obs = obs_s2dv, fun_analog = "max",
                        lat_dim = 'latitude', lon_dim = 'longitude',
                        ncores = 4, grid_exp = attr(exp, "Files")[1,1,1])




