library(startR)
library(s2dv)
library(CSTools)
library(lubridate)
library(multiApply)
source('R/Interpolation.R')
source('R/Utils.R')
source('R/Intbc.R')

target_grid <- '/esarchive/recon/ecmwf/era5/monthly_mean/tas_f1h/tas_200002.nc'

lonmin <- -11.5
lonmax <- 5.35
latmin <- 35.1
latmax <- 44.1
extra <- 1

sdates <- format(ymd("20000201") + rep(years(0:20), each=1),"%Y%m%d")
sdates_fcst <- format(seq(ymd("20230201"), ymd("20230401"), "1 month"), "%Y%m%d")

#---------------------------
# Observations
#---------------------------
# ERA5-Land observations: high-resolution observations
obs <- startR::Start(dat = '/esarchive/recon/ecmwf/era5land/monthly_mean/$var$_f1h/$var$_$sdate$.nc',
                     var = 'tas', time = indices(1), lat = values(list(latmin - extra, latmax + extra)),
                     lat_reorder = Sort(decreasing = FALSE), lon = values(list(lonmin - extra, lonmax + extra)),
                     lon_reorder = CircularSort(-180, 180), sdate = format(ymd(sdates), "%Y%m"), 
                     synonims = list(var = c('var','variable'), lon = c('lon', 'longitude'), 
                                     lat = c('lat', 'latitude')), return_vars = list(lat = 'dat', lon = 'dat'),
                     num_procs = 1, retrieve = TRUE)

#---------------------------
# Model
#---------------------------
# ecmwf- system5c3s : coarse resolution
exp <- startR::Start(dat = '/esarchive/exp/ecmwf/system5c3s/monthly_mean/$var$_f6h/$var$_$sdate$.nc',
                     var = 'tas', time = indices(1), member = 'all', sdate = sdates,
                     lat = values(list(latmin - extra, latmax + extra)), lat_reorder = Sort(decreasing = FALSE), 
                     lon = values(list(lonmin - extra, lonmax + extra)), lon_reorder = CircularSort(-180, 180),
                     synonims = list(var = c('var','variable'), lon = c('lon', 'longitude'), 
                                     lat = c('lat', 'latitude'), member = c('member','ensemble')),
                     return_vars = list(lat = 'dat', lon = 'dat'),
                     num_procs = 1, retrieve = TRUE)

#---------------------------
# Forecast
#---------------------------
# ecmwf- system5c3s : coarse resolution
fcst <- startR::Start(dat = '/esarchive/exp/ecmwf/system51c3s/monthly_mean/$var$_f6h/$var$_$sdate$.nc',
                      var = 'tas', time = indices(1), member = 'all', sdate = sdates_fcst,
                      lat = values(list(latmin - extra, latmax + extra)), lat_reorder = Sort(decreasing = FALSE),
                      lon = values(list(lonmin - extra, lonmax + extra)), lon_reorder = CircularSort(-180, 180),
                      synonims = list(var = c('var','variable'), lon = c('lon', 'longitude'),
                                      lat = c('lat', 'latitude'), member = c('member','ensemble')),
                      return_vars = list(lat = 'dat', lon = 'dat'),
                      num_procs = 1, retrieve = TRUE)

obss2dv <- as.s2dv_cube(obs)
exps2dv <- as.s2dv_cube(exp)
fcsts2dv <- as.s2dv_cube(fcst)

#------------------------------------
# Downscaling with different methods
#------------------------------------

down_1 <- Intbc(exp = exp, obs = obs, exp_lats = attr(exp, "Variables")$dat1$lat, 
                exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs, "Variables")$dat1$lat, 
                obs_lons = attr(obs, "Variables")$dat1$lon, target_grid = target_grid, int_method = 'con', 
                bc_method = 'evmos', ncores = 4)

down_2 <- Intbc(exp = exp, obs = obs, exp_lats = attr(exp, "Variables")$dat1$lat, 
                exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs, "Variables")$dat1$lat, 
                obs_lons = attr(obs, "Variables")$dat1$lon, target_grid = target_grid, int_method = 'bic', 
                bc_method = 'quantile_mapping', ncores = 4)

down_3 <- Intbc(exp = exp, obs = obs, exp_lats = attr(exp, "Variables")$dat1$lat, 
                exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs, "Variables")$dat1$lat, 
                obs_lons = attr(obs, "Variables")$dat1$lon, target_grid = target_grid, int_method = 'bil', 
                bc_method = 'bias', ncores = 4)

down_4 <- Intbc(exp = exp, obs = obs, exp_lats = attr(exp, "Variables")$dat1$lat, 
                exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs, "Variables")$dat1$lat, 
                obs_lons = attr(obs, "Variables")$dat1$lon, target_grid = target_grid, int_method = 'nn', 
                bc_method = 'mse_min', ncores = 4)

down_5 <- Intbc(exp = exp, obs = obs, exp_lats = attr(exp, "Variables")$dat1$lat, 
                exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs, "Variables")$dat1$lat, 
                obs_lons = attr(obs, "Variables")$dat1$lon, target_grid = target_grid, int_method = 'con', 
                bc_method = 'crps_min', ncores = 4)

down_6 <- Intbc(exp = exp, obs = obs, exp_lats = attr(exp, "Variables")$dat1$lat, 
                exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs, "Variables")$dat1$lat, 
                obs_lons = attr(obs, "Variables")$dat1$lon, target_grid = target_grid, int_method = 'bil', 
                bc_method = 'rpc-based', ncores = 4)

# not working check ----------------
down_2 <- Intbc(exp = exp, obs = obs, exp_lats = attr(exp, "Variables")$dat1$lat, 
                exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs, "Variables")$dat1$lat, 
                obs_lons = attr(obs, "Variables")$dat1$lon, target_grid = target_grid, int_method = 'bil', 
                bc_method = 'dynamical_bias', ncores = 4)
#-----------------------------------

cstdown_1 <- CST_Intbc(exp = exp, obs = obs, target_grid = target_grid, int_method = 'con', 
                       bc_method = 'simple_bias', ncores = 4)

#------------------------------------
# Downscaling to point locations
#------------------------------------
points <- list(lat = c(36.1, 38.7), lon = c(-1.9, 0.8))

down_points_1 <- Intbc(exp = exp, obs = obs, 
                       exp_lats = attr(exp, "Variables")$dat1$lat, exp_lons = attr(exp, "Variables")$dat1$lon,
                       obs_lats = attr(obs, "Variables")$dat1$lat, obs_lons = attr(obs, "Variables")$dat1$lon, 
                       target_grid = target_grid, 
                       bc_method = 'quantile_mapping', points = points, 
                       source_file_exp = "/esarchive/exp/ecmwf/system5c3s/monthly_mean/tas_f6h/tas_20000201.nc", 
                       source_file_obs = "/esarchive/recon/ecmwf/era5/monthly_mean/tas_f1h-r1440x721cds/tas_200002.nc", 
                       method_point_interp = 'bilinear')

down_points_2 <- Intbc(exp = exp, obs = obs, 
                       exp_lats = attr(exp, "Variables")$dat1$lat, exp_lons = attr(exp, "Variables")$dat1$lon, 
                       obs_lats = attr(obs, "Variables")$dat1$lat, obs_lons = attr(obs, "Variables")$dat1$lon,
                       source_file_exp = "/esarchive/exp/ecmwf/system5c3s/monthly_mean/tas_f6h/tas_20000201.nc", 
                       source_file_obs = "/esarchive/recon/ecmwf/era5/monthly_mean/tas_f1h-r1440x721cds/tas_200002.nc", 
                       method_point_interp = 'nearest')

down_points_3 <- Intbc(exp = exp, obs = obs, 
                       exp_lats = attr(exp, "Variables")$dat1$lat, exp_lons = attr(exp, "Variables")$dat1$lon, 
                       obs_lats = attr(obs, "Variables")$dat1$lat, obs_lons = attr(obs, "Variables")$dat1$lon, 
                       target_grid = target_grid, 
                       bc_method = 'mse_min', points = points, 
                       source_file_exp = "/esarchive/exp/ecmwf/system5c3s/monthly_mean/tas_f6h/tas_20000201.nc", 
                       source_file_obs = "/esarchive/recon/ecmwf/era5/monthly_mean/tas_f1h-r1440x721cds/tas_200002.nc", 
                       method_point_interp = '9point')

down_points_4 <- Intbc(exp = exp, obs = obs, 
                       exp_lats = attr(exp, "Variables")$dat1$lat, exp_lons = attr(exp, "Variables")$dat1$lon, 
                       obs_lats = attr(obs, "Variables")$dat1$lat, obs_lons = attr(obs, "Variables")$dat1$lon, 
                       target_grid = target_grid, 
                       bc_method = 'crps_min', points = points, 
                       source_file_exp = "/esarchive/exp/ecmwf/system5c3s/monthly_mean/tas_f6h/tas_20000201.nc", 
                       source_file_obs = "/esarchive/recon/ecmwf/era5/monthly_mean/tas_f1h-r1440x721cds/tas_200002.nc", 
                       method_point_interp = 'invdist4nn')

down_points_5 <- Intbc(exp = exp, obs = obs, 
                       exp_lats = attr(exp, "Variables")$dat1$lat, exp_lons = attr(exp, "Variables")$dat1$lon, 
                       obs_lats = attr(obs, "Variables")$dat1$lat, obs_lons = attr(obs, "Variables")$dat1$lon, 
                       target_grid = target_grid, 
                       bc_method = 'rpc-based', points = points, 
                       source_file_exp = "/esarchive/exp/ecmwf/system5c3s/monthly_mean/tas_f6h/tas_20000201.nc", 
                       source_file_obs = "/esarchive/recon/ecmwf/era5/monthly_mean/tas_f1h-r1440x721cds/tas_200002.nc", 
                       method_point_interp = 'SW')


cstdown_points_1  <- CST_Intbc(exp = exps2dv, obs = obss2dv, points = points, method_point_interp = 'bilinear',
                               target_grid = target_grid, bc_method = 'quantile_mapping', ncores = 4, loocv = FALSE)

#------------------------------------
# Downscaling to point locations given point observations
#------------------------------------
points <- list(lat = c(36.1, 38.7), lon = c(-1.9, 0.8))
obs_point <- Subset(obs$data, along = c("lat", "lon"), indices = list(1:2,1), drop = FALSE) 
obs_point <- MergeDims(data = obs_point, merge_dims = c("lat", "lon"), rename_dim = 'location') 

down_obspoints_1 <- Intbc(exp = exp, obs = obs_point, exp_lats = attr(exp, "Variables")$dat1$lat,
                          exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = points$lat,
                          obs_lons = points$lon, points = points, method_point_interp = 'bilinear',
                          source_file_exp = attr(exp, "Files")[1,1,1], source_file_obs = NULL,
                          target_grid = target_grid, bc_method = 'quantile_mapping', ncores = 4)

cstdown_obspoints_1  <- CST_Intbc(exp = exps2dv, obs = obss2dv, points = points, method_point_interp = 'bilinear',
                               target_grid = target_grid, bc_method = 'quantile_mapping', ncores = 4, loocv = FALSE)
#-----------------------------------------------
# Downscaling to specified region
#-----------------------------------------------
crplonmin <- -1
crplonmax <- 4 
crplatmin <- 39
crplatmax <- 44

region_ext = c(crplonmin, crplonmax, crplatmin, crplatmax)

down_region <- Intbc(exp = exp, obs = obs, 
                     exp_lats = attr(exp, "Variables")$dat1$lat, exp_lons = attr(exp, "Variables")$dat1$lon,
                     obs_lats = attr(obs, "Variables")$dat1$lat, obs_lons = attr(obs, "Variables")$dat1$lon, 
                     int_method = 'con',
                     target_grid = target_grid, region = region_ext,
                     bc_method = 'quantile_mapping',
                     source_file_obs = "/esarchive/recon/ecmwf/era5/monthly_mean/tas_f1h-r1440x721cds/tas_200002.nc", 
                     )

cstdown_region <- CST_Intbc(exp = exps2dv, obs = obss2dv, points = points, method_point_interp = 'bilinear', 
                            int_method = 'con', region = region_ext,
                            target_grid = target_grid, bc_method = 'quantile_mapping', ncores = 4)

#---------------------------------------------------
# Downscaling forecasts using hindcasts to train
#---------------------------------------------------
down_1_sbc <- Intbc(exp = exp, obs = obs, exp_cor = fcst, exp_lats = attr(exp, "Variables")$dat1$lat,
                    exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs, "Variables")$dat1$lat,
                    obs_lons = attr(obs, "Variables")$dat1$lon, target_grid = target_grid, int_method = 'con',
                    bc_method = 'evmos', ncores = 4)


down_2_sbc <- CST_Intbc(exp = exps2dv, obs = obss2dv,  target_grid = target_grid, int_method = 'con',
                        bc_method = 'evmos', ncores = 4)


#------------------------------------
# Downscaling for daily data
#------------------------------------

#---------------------------
# Observations
#---------------------------
# ERA5-Land observations: high-resolution observations
lonmin <- -11.5
lonmax <- 5.35
latmin <- 35.1
latmax <- 44.1
extra <- 1

obs <- startR::Start(dat = '/esarchive/recon/ecmwf/era5land/daily_mean/$var$_f1h/$var$_$sdate$.nc',
                          var = 'tas', time = indices(1:28), latitude = values(list(latmin - extra, latmax + extra)),
                          sdate = format(ymd(sdates), "%Y%m"), latitude_reorder = Sort(decreasing = FALSE),
                          longitude = values(list(lonmin - extra, lonmax + extra)),
                          longitude_reorder = CircularSort(-180, 180),
                          synonims = list(var = c('var','variable'), longitude = c('lon', 'longitude'),
                                          latitude = c('lat', 'latitude')), return_vars = list(latitude = 'dat', longitude = 'dat'),
                          num_procs = 1, retrieve = TRUE)

#---------------------------
# Model
#---------------------------
# ecmwf- system5c3s : coarse resolution
exp <- startR::Start(dat = '/esarchive/exp/ecmwf/system5c3s/daily_mean/$var$_f6h/$var$_$sdate$.nc',
                     var = 'tas', time = indices(1:28), member = indices(1:3), sdate = format(ymd(sdates), "%Y%m%d"),
                     latitude = values(list(latmin - extra , latmax + extra)), latitude_reorder = Sort(decreasing = FALSE), 
                     longitude = values(list(lonmin - extra, lonmax + extra)), longitude_reorder = CircularSort(-180, 180),
                     synonims = list(var = c('var','variable'), longitude = c('lon', 'longitude'), 
                                     latitude = c('lat', 'latitude'), member = c('member','ensemble')),
                     return_vars = list(latitude = 'dat', longitude = 'dat'),
                     num_procs = 1, retrieve = TRUE)


obss2dv <- as.s2dv_cube(obs)
exps2dv <- as.s2dv_cube(exp)

#------------------------------------
# Downscaling with different methods
#------------------------------------

down_1_sbc <- Intbc(exp = exp, obs = obs, 
                    obs_lats = attr(obs, "Variables")$dat1$lat, obs_lons = attr(obs, "Variables")$dat1$lon, 
                    exp_lats = attr(exp, "Variables")$dat1$lat, exp_lons = attr(exp, "Variables")$dat1$lon, 
                    target_grid = target_grid, int_method = 'con', bc_method = 'quantile_mapping', ncores = 4,
                    lat_dim = 'latitude', lon_dim = 'longitude')

# Parameter 'indices' has different length of parameter data in the dimension supplied in 'split_dim'.problem in last lines after if(daily)
down_2_sbc <- CST_Intbc(exp = exp, obs = obs, target_grid = target_grid, int_method = 'con', 
                        bc_method = 'simple_bias', ncores = 4)

