library(startR)
library(s2dv)
library(CSTools)
library(lubridate)
library(multiApply)
library(easyVerification)
source('R/Interpolation.R')
source('R/Utils.R')
source('R/LogisticReg.R')

target_grid <- '/esarchive/recon/ecmwf/era5/monthly_mean/tas_f1h/tas_200002.nc'

lonmin <- -11.5
lonmax <- 5.35
latmin <- 35.1
latmax <- 44.1

sdates <- format(ymd("20000701") + rep(years(0:20), each=1),"%Y%m%d")
sdates_fcst <- c('20230201', '20230401')

#---------------------------
# Observations
#---------------------------
# ERA5-Land observations: high-resolution observations
obs <- startR::Start(dat = '/esarchive/recon/ecmwf/era5land/monthly_mean/$var$_f1h/$var$_$sdate$.nc',
                     var = 'tas', time = indices(1), lat = values(list(latmin - extra, latmax + extra)),
                     lat_reorder = Sort(decreasing = FALSE), lon = values(list(lonmin - extra, lonmax + extra)),
                     lon_reorder = CircularSort(-180, 180), sdate = format(ymd(sdates), "%Y%m"), 
                     synonims = list(var = c('var','variable'), lon = c('lon', 'longitude'), 
                                     lat = c('lat', 'latitude')), return_vars = list(lat = 'dat', lon = 'dat'),
                     num_procs = 1, retrieve = TRUE)

#---------------------------
# Model
#---------------------------
# ecmwf- system5c3s : coarse resolution
exp <- startR::Start(dat = '/esarchive/exp/ecmwf/system5c3s/monthly_mean/$var$_f6h/$var$_$sdate$.nc',
                     var = 'tas', time = indices(1), member = 'all', sdate = sdates,
                     lat = values(list(latmin - extra, latmax + extra)), lat_reorder = Sort(decreasing = FALSE), 
                     lon = values(list(lonmin - extra, lonmax + extra)), lon_reorder = CircularSort(-180, 180),
                     synonims = list(var = c('var','variable'), lon = c('lon', 'longitude'), 
                                     lat = c('lat', 'latitude'), member = c('member','ensemble')),
                     return_vars = list(lat = 'dat', lon = 'dat'),
                     num_procs = 1, retrieve = TRUE)

#---------------------------
# Forecast
#---------------------------
# ecmwf- system5c3s : coarse resolution
fcst <- startR::Start(dat = '/esarchive/exp/ecmwf/system51c3s/monthly_mean/$var$_f6h/$var$_$sdate$.nc',
                      var = 'tas', time = indices(1), member = 'all', sdate = sdates_fcst,
                      lat = values(list(latmin - extra, latmax + extra)), lat_reorder = Sort(decreasing = FALSE),
                      lon = values(list(lonmin - extra, lonmax + extra)), lon_reorder = CircularSort(-180, 180),
                      synonims = list(var = c('var','variable'), lon = c('lon', 'longitude'),
                                      lat = c('lat', 'latitude'), member = c('member','ensemble')),
                      return_vars = list(lat = 'dat', lon = 'dat'),
                      num_procs = 1, retrieve = TRUE)

obss2dv <- as.s2dv_cube(obs)
exps2dv <- as.s2dv_cube(exp)
fcsts2dv <- as.s2dv_cube(fcst)



#--------------------------------------------
# Downscaling with Logisticregression
#--------------------------------------------

down_1 <- LogisticReg(exp = exp, obs = obs, exp_lats = attr(exp, "Variables")$dat1$lat, 
                      exp_lons = attr(exp, "Variables")$dat1$lon, 
                      obs_lats = attr(obs, "Variables")$dat1$lat, 
                      obs_lons = attr(obs, "Variables")$dat1$lon, 
                      probs_cat = 1:4/5, target_grid = target_grid, int_method = "bil", loocv = TRUE,
                      return_most_likely_cat = TRUE)


down_2 <- LogisticReg(exp = exp, obs = obs, exp_lats = attr(exp, "Variables")$dat1$lat, 
                      exp_lons = attr(exp, "Variables")$dat1$lon, 
                      obs_lats = attr(obs, "Variables")$dat1$lat, 
                      obs_lons = attr(obs, "Variables")$dat1$lon, log_reg_method = "ens_mean",
                      probs_cat = 1:4/5, target_grid = target_grid, int_method = "con", loocv = TRUE,
                      return_most_likely_cat = TRUE)

down_3 <- LogisticReg(exp = exp, obs = obs, exp_lats = attr(exp, "Variables")$dat1$lat, 
                      exp_lons = attr(exp, "Variables")$dat1$lon, 
                      obs_lats = attr(obs, "Variables")$dat1$lat, 
                      obs_lons = attr(obs, "Variables")$dat1$lon, log_reg_method = "ens_mean_sd",
                      probs_cat = 1:4/5, target_grid = target_grid, int_method = "bic", loocv = FALSE,
                      return_most_likely_cat = TRUE)

down_4 <- LogisticReg(exp = exp, obs = obs, exp_lats = attr(exp, "Variables")$dat1$lat, 
                      exp_lons = attr(exp, "Variables")$dat1$lon, 
                      obs_lats = attr(obs, "Variables")$dat1$lat, 
                      obs_lons = attr(obs, "Variables")$dat1$lon, log_reg_method = "sorted_members",
                      probs_cat = 1:4/5, target_grid = target_grid, int_method = "nn", loocv = TRUE,
                      return_most_likely_cat = FALSE)

cstdown_1 <- CST_LogisticReg(exp = exps2dv, obs = obss2dv, probs_cat = 1:2/3,  target_grid = target_grid,
                             int_method = "bil", log_reg_method = "ens_mean",
                             return_most_likely_cat = TRUE, loocv = TRUE)

#----------------------------------
# Downscaling to point locations
#----------------------------------
points <- list(lat = c(36.1, 38.7), lon = c(-1.9, 0.8))

down_points_1 <- LogisticReg(exp = exp, obs = obs, exp_lats = attr(exp, "Variables")$dat1$lat,
                             exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs, "Variables")$dat1$lat,
                             obs_lons = attr(obs, "Variables")$dat1$lon, target_grid = target_grid, 
                             points = points, method_point_interp = 'bilinear', int_method = 'bil', 
                             source_file_exp = attr(exp, "Files")[1,1,1], source_file_obs = attr(obs, "Files")[1,1,1],
                             loocv = TRUE, ncores = 4)

down_points_2 <- LogisticReg(exp = exp, obs = obs, exp_lats = attr(exp, "Variables")$dat1$lat,
                             exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs, "Variables")$dat1$lat,
                             obs_lons = attr(obs, "Variables")$dat1$lon, target_grid = target_grid, 
                             points = points, method_point_interp = 'nearest', int_method = 'con', 
                             source_file_exp = attr(exp, "Files")[1,1,1], source_file_obs = attr(obs, "Files")[1,1,1],
                             loocv = TRUE, ncores = 4)

down_points_3 <- LogisticReg(exp = exp, obs = obs, exp_lats = attr(exp, "Variables")$dat1$lat,
                             exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs, "Variables")$dat1$lat,
                             obs_lons = attr(obs, "Variables")$dat1$lon, target_grid = target_grid, 
                             points = points, method_point_interp = '9point', int_method = 'nn', 
                             source_file_exp = attr(exp, "Files")[1,1,1], source_file_obs = attr(obs, "Files")[1,1,1],
                             loocv = TRUE, ncores = 4)

down_points_4 <- LogisticReg(exp = exp, obs = obs, exp_lats = attr(exp, "Variables")$dat1$lat,
                             exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs, "Variables")$dat1$lat,
                             obs_lons = attr(obs, "Variables")$dat1$lon, target_grid = target_grid, 
                             points = points, method_point_interp = 'invdist4nn', int_method = 'bic', 
                             source_file_exp = attr(exp, "Files")[1,1,1], source_file_obs = attr(obs, "Files")[1,1,1],
                             loocv = TRUE, ncores = 4)

cstdown_points_1 <- CST_LogisticReg(exp = exps2dv, obs = obss2dv, target_grid = target_grid, 
                                    points = points, method_point_interp = 'SE', int_method = 'bil', 
                                    loocv = TRUE, ncores = 4)

#----------------------------------
# Downscaling to point locations given point observations
#----------------------------------
points <- list(lat = c(36.1, 38.7), lon = c(-1.9, 0.8))
obs_point <- Subset(obs, along = c("lat", "lon"), indices = list(1:2,1), drop = FALSE)
obs_point <- MergeDims(data = obs_point, merge_dims = c("lat", "lon"), rename_dim = 'location')

down_obspoint <- LogisticReg(exp = exp, obs = obs_point, exp_lats = attr(exp, "Variables")$dat1$lat,
                             exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = points$lat, obs_lons = points$lon, 
                             target_grid = target_grid,  points = points, 
                             method_point_interp = 'bilinear', int_method = 'bil', 
                             source_file_exp = attr(exp, "Files")[1,1,1], source_file_obs = attr(obs, "Files")[1,1,1], 
                             loocv = TRUE, ncores = 1)

cstdown_obspoints <- CST_LogisticReg(exp = exps2dv, obs = obss2dv, target_grid = target_grid, 
                                     points = points, method_point_interp = 'nearest',
                                     int_method = "nearest", loocv = TRUE, ncores = 4)

#----------------------------------
# Downscaling to specified region
#----------------------------------
crplonmin <- -1
crplonmax <- 4 
crplatmin <- 39
crplatmax <- 44

region_ext = c(crplonmin, crplonmax, crplatmin, crplatmax)

down_region <- LogisticReg(exp = exp, obs = obs, exp_lats = attr(exp, "Variables")$dat1$lat,
                           exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs, "Variables")$dat1$lat,
                           obs_lons = attr(obs, "Variables")$dat1$lon, target_grid = target_grid, 
                           int_method = "bilinear", loocv = TRUE, ncores = 4, region = region_ext)

cstdown_region <- CST_LogisticReg(exp = exps2dv, obs = obss2dv, target_grid = target_grid, 
                                  region = region_ext, int_method = "bilinear",  log_reg_method = "sorted_members",
                                  loocv = TRUE, ncores = 4)

#------------------------------------
# Downscaling forecasts using hindcasts to train
#------------------------------------

down_fcst <- LogisticReg(exp = exp, obs = obs, exp_cor = fcst, exp_lats = attr(exp, "Variables")$dat1$lat,
                         exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs, "Variables")$dat1$lat,
                         obs_lons = attr(obs, "Variables")$dat1$lon, target_grid = target_grid, 
                         log_reg_method = "ens_mean_sd", int_method = "bic", 
                         loocv = TRUE, ncores = 4)

cstdown_fcst <- CST_LogisticReg(exp = exps2dv, obs = obss2dv, exp_cor = fcsts2dv, 
                                target_grid = target_grid, int_method = "bilinear",  
                                log_reg_method = "ens_mean", loocv = TRUE, ncores = 4) 



#------------------------------------
# Downscaling for daily data
#------------------------------------

#---------------------------
# Observations
#---------------------------
# ERA5-Land observations: high-resolution observations
lonmin <- -11.5
lonmax <- 5.35
latmin <- 35.1
latmax <- 44.1
extra <- 1

obs <- startR::Start(dat = '/esarchive/recon/ecmwf/era5land/daily_mean/$var$_f1h/$var$_$sdate$.nc',
                     var = 'tas', time = indices(1:28), latitude = values(list(latmin - extra, latmax + extra)),
                     sdate = format(ymd(sdates), "%Y%m"), latitude_reorder = Sort(decreasing = FALSE),
                     longitude = values(list(lonmin - extra, lonmax + extra)),
                     longitude_reorder = CircularSort(-180, 180),
                     synonims = list(var = c('var','variable'), longitude = c('lon', 'longitude'),
                                     latitude = c('lat', 'latitude')), return_vars = list(latitude = 'dat', longitude = 'dat'),
                     num_procs = 1, retrieve = TRUE)

#---------------------------
# Model
#---------------------------
# ecmwf- system5c3s : coarse resolution
exp <- startR::Start(dat = '/esarchive/exp/ecmwf/system5c3s/daily_mean/$var$_f6h/$var$_$sdate$.nc',
                     var = 'tas', time = indices(1:28), member = indices(1:3), sdate = format(ymd(sdates), "%Y%m%d"),
                     latitude = values(list(latmin - extra , latmax + extra)), latitude_reorder = Sort(decreasing = FALSE), 
                     longitude = values(list(lonmin - extra, lonmax + extra)), longitude_reorder = CircularSort(-180, 180),
                     synonims = list(var = c('var','variable'), longitude = c('lon', 'longitude'), 
                                     latitude = c('lat', 'latitude'), member = c('member','ensemble')),
                     return_vars = list(latitude = 'dat', longitude = 'dat'),
                     num_procs = 1, retrieve = TRUE)


obss2dv <- as.s2dv_cube(obs)
exps2dv <- as.s2dv_cube(exp)

#------------------------------------
# Downscaling with Logisticregression
#------------------------------------

down_daily <- LogisticReg(exp = exp, obs = obs, exp_lats = attr(exp, "Variables")$dat1$lat, 
                          exp_lons = attr(exp, "Variables")$dat1$lon, 
                          obs_lats = attr(obs, "Variables")$dat1$lat, 
                          obs_lons = attr(obs, "Variables")$dat1$lon, 
                          probs_cat = 1:4/5, target_grid = target_grid, int_method = "bil", loocv = TRUE,
                          lon_dim='longitude', lat_dim='latitude',
                          return_most_likely_cat = TRUE)

cstdown_daily <- CST_LogisticReg(exp = exps2dv, obs = obss2dv, probs_cat = 1:2/3,  target_grid = target_grid,
                                 int_method = "bil", log_reg_method = "ens_mean_sd",
                                 lon_dim='longitude', lat_dim='latitude',
                                 return_most_likely_cat = TRUE, loocv = TRUE)


