source('R/Interpolation.R')
require(testthat)
require(CSTools)
require(s2dv)

set.seed(12)
exp <- rnorm(500)
dim(exp) <- c(member = 5, lat = 4, lon = 5, sdate = 5, time  = 1)
lons <- 1:5
lats <- 1:4
exps2dv <- suppressWarnings(s2dv_cube(data = exp, lat = lats, lon = lons)) 

exp1 <- exp
names(dim(exp1)) <- c("member", "latitude", "longitude", "sdate", "time")
exp1s2dv <- suppressWarnings(s2dv_cube(data = exp1, lat = lats, lon = lons)) 

lats2 <- 88:92

test_that("1. Input checks", {
  
  expect_error(
    CST_Interpolation(exp = 1),
    "Parameter 'exp' must be of the class 's2dv_cube'")
  
  expect_error(
    CST_Interpolation(exp = exp1s2dv),
    paste0("The name of the latitude/longitude dimensions in 'exp$data' must match the parametres 'lat_dim' and 'lon_dim'"),
    fixed = TRUE
  )
  
})

test_that("2. Input checks", {
  
  expect_error(
    Interpolation(exp = exp, lats = lats, lons = lons, method_remap = 1),
    paste0("Parameter 'method_remap' must be of the class 'character'"))  
  
  expect_error(
    Interpolation(exp = exp, lats = lats, lons = lons, method_point_interp = 1),
    paste0("Parameter 'method_point_interp' must be of the class 'character'")
  )
  
  expect_error(
    Interpolation(exp = exp1, lats = lats, lons = lons),
    paste0("Missing longitude dimension in 'exp', or does not match the parameter 'lon_dim'"))  
  
  expect_error(
    Interpolation(exp = exp, lats = lats2, lons = lons),
    paste0("Out-of-range latitudes have been found. Latitudes must range from -90 to 90"))  
  
  expect_error(
    Interpolation(exp = exp, lats = lats, lons = lons, points = c(1,2)),
    paste0("Parameter 'points' must be a list of two elements containing the point latitudes and longitudes."))  
  
  expect_error(
    Interpolation(exp = exp, lats = lats, lons = lons, points = list(lat = c(1,2), lon = c(3,4))),
    paste0("Parameter 'method_point_interp' must be a character vector indicating the interpolation method.",
           " Accepted methods are nearest, bilinear, 9point, invdist4nn, NE, NW, SE, SW"),
    fixed = TRUE)  
  
  expect_error(
    Interpolation(exp = exp, lats = lats, lons = lons, points = list(lat = c(1,2), lon = c(3,4)),
                  method_point_interp = 'near'),
    paste0("Parameter 'method_point_interp' must be a character vector indicating the interpolation method. ",
           "Accepted methods are nearest, bilinear, 9point, invdist4nn, NE, NW, SE, SW"),
    fixed = TRUE)  
  
  expect_error(
    Interpolation(exp = exp, lats = lats, lons = lons, points = list(lat = c(1,2), lon = c(3,4), alt = c(1,2)),
                  method_point_interp = 'nearest'),
    paste0("'points' must be a lis of two elements containing the point ",
           "latitudes and longitudes in the form 'points$lat', 'points$lon'"),
    fixed = TRUE)
  
  expect_error(
    Interpolation(exp = exp, lats = lats, lons = lons, points = list(lats = c(1,2), lons = c(3,4)),
                  method_point_interp = 'nearest'),
    paste0("The names of the elements in the list 'points' must coincide with the parametres ", 
           "'lat_dim' and 'lon_dim'"))
  
  expect_error(
    Interpolation(exp = exp, lats = lats, lons = lons, points = list(lat = c(1,2,3), lon = c(3,4)),
                  method_point_interp = 'nearest'),
    paste0("The number of latitudes and longitudes must match"))
  
  expect_error(
    Interpolation(exp = exp, lats = lats, lons = lons, points = list(lat = c(91,2), lon = c(3,4)),
                  method_point_interp = 'nearest'),
    paste0("Out-of-range latitudes have been found in 'points'. Latitudes must range from ",
           "-90 to 90"))
  
  expect_error(
    Interpolation(exp = exp, lats = lats, lons = lons, points = list(lat = c(1,2), lon = c(3,4)),
                  method_point_interp = 'nearest'),
    paste0("No source file found. Source file must be provided in the parameter 'source_file'."))
  
  expect_error(
    Interpolation(exp = exp, lats = lats, lons = lons),
    paste0("Parameter 'method_remap' must be a character vector indicating the interpolation method. ",
           "Accepted methods are con, bil, bic, nn, con2"))
  
  expect_error(
    Interpolation(exp = exp, lats = lats, lons = lons, method_remap = 'bil'),
    paste0("Parameter 'target_grid' can be either a path ",
           "to another NetCDF file which to read the target grid from (a single grid must be ",
           "defined in such file) or a character vector indicating the coarse grid to ",
           "be passed to CDO, and it must be a grid recognised by CDO or a NetCDF file."),
    fixed = TRUE)
  
  expect_error(
    Interpolation(exp = exp, lats = lats, lons = lons, method_remap = 'bil', target_grid = 'r1280x640',
                  ncores = -1),
    paste0("Parameter 'ncores' must be a positive integer."))
  
  expect_error(
    suppressWarnings(Interpolation(exp = exp, lats = lats, lons = lons,
                                   method_remap = 'bil', target_grid = 'r1280x640',
                                   points = list(lat = c(1,2), lon = c(3,46)),
                                   method_point_interp = 'nearest', source_file = 'null.nc')),
    paste0("At least one of the points lies outside the model region"))
})

test_that("3. Output checks", {
  
  res <- suppressWarnings(CST_Interpolation(exp = exps2dv, method_remap = 'bil', 
                                            target_grid = 'r1280x640'))
  
  expect_equal(length(res$exp), 4)
  expect_equal(res$obs, NULL)
  expect_equal(class(res), "list")
  expect_equal(class(res$exp), "s2dv_cube")
  expect_equal(as.numeric(dim(res$exp$data)[names(dim(res$exp$data)) == 'lat']), length(res$exp$coords$lat))
  expect_equal(as.numeric(dim(res$exp$data)[names(dim(res$exp$data)) == 'lon']), length(res$exp$coords$lon))
  
})

test_that("4. Output checks", {
  
  res1 <- suppressWarnings(
    Interpolation(exp = exp, lats = lats, lons = lons,
                  method_remap = 'bil', target_grid = 'r1280x640'))
  
  expect_equal(length(res1), 4)
  expect_equal(res1$obs, NULL)
  expect_equal(class(res1), "list")
  expect_equal(class(res1$data), "array")
  expect_equal(as.numeric(dim(res1$data)[names(dim(res1$data)) == 'lat']), length(res1$lat))
  expect_equal(as.numeric(dim(res1$data)[names(dim(res1$data)) == 'lon']), length(res1$lon))
  expect_equal(round(res1$data[1:3],3), c(-1.003,  1.118, -0.664))
  
  # point location
  # points <- list(lat = c(36.1, 38.7), lon = c(-1.9, 0.8))
  # res2 <- suppressWarnings(
  #   Interpolation(exp = exp, lats = attr(exp, "Variables")$dat1$lat, 
  #                 lons = attr(exp, "Variables")$dat1$lon,
  #                 points = points, method_point_interp = 'bilinear', 
  #                 source_file = attr(exp, "Files")[1,1,1])
  
  # expect_equal(as.numeric(res2$lat), points[["lat"]])
  # expect_equal(as.numeric(res2$lon), points[["lon"]])
  
  res2 <- suppressWarnings(
    Interpolation(exp = exp, lats = lats, lons = lons,
                  method_remap = 'con', target_grid = 'r1280x640'))
  expect_equal(round(res2$data[1:3],3), c(-1.481,  1.577, -0.957))
  
  
  
})


